<?php

namespace IPS\easypopup\extensions\core\FileStorage;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * Editor Extension: Icons
 */
class _Image
{
	/**
	 * Count stored files
	 *
	 * @return	int
	 */
	public function count()
	{
		return \IPS\Db::i()->select( 'COUNT(*)', 'easypopup_popups', "espp_image IS NOT NULL AND espp_image != ''" )->first();
	}
	
	/**
	 * Move stored files
	 *
	 * @param	int			$offset					This will be sent starting with 0, increasing to get all files stored by this extension
	 * @param	int			$storageConfiguration	New storage configuration ID
	 * @param	int|NULL	$oldConfiguration		Old storage configuration ID
	 * @throws	\Underflowexception				When file record doesn't exist. Indicating there are no more files to move
	 * @return	void
	 */
	public function move( $offset, $storageConfiguration, $oldConfiguration=NULL )
	{
		$popup = \IPS\Db::i()->select( '*', 'easypopup_popups', "espp_image IS NOT NULL AND espp_image != ''", 'espp_id', array( $offset, 1 ) )->first();
		
		try
		{
			$image = \IPS\File::get( $oldConfiguration ?: 'easypopup_Image', $popup['espp_image'] )->move( $storageConfiguration );
		
			\IPS\Db::i()->update( 'easypopup_popups', array( 'espp_image' => (string) $image ), array( 'espp_id=?', $popup['espp_id'] ) );	
		}
		catch( \Exception $e )
		{
			/* Any issues are logged and the \IPS\Db::i()->update not run as the exception is thrown */
		}		
	}

	/**
	 * Check if a file is valid
	 *
	 * @param	\IPS\Http\Url	$file		The file to check
	 * @return	bool
	 */
	public function isValidFile( $file )
	{
		try
		{
			\IPS\Db::i()->select( '*', 'easypopup_popups', array( 'espp_image=?', (string) $file ) )->first();
			return TRUE;
		}
		catch ( \UnderflowException $e )
		{
			return FALSE;
		}
	}

	/**
	 * Delete all stored files
	 *
	 * @return	void
	 */
	public function delete()
	{
		foreach( \IPS\Db::i()->select( '*', 'easypopup_popups', "espp_image IS NOT NULL AND espp_image != ''" ) as $popup )
		{
			try
			{
				\IPS\File::get( 'easypopup_Image', $popup['espp_image'] )->delete();			
			}
			catch( \Exception $e ){}
		}
	}
}