/*===================================================================================================================
 * @name: bPopup
 * @type: jQuery
 * @author: (c) Bjoern Klinggaard - @bklinggaard
 * @demo: http://dinbror.dk/bpopup
 * @version: 0.11.0
 * @requires jQuery 1.4.3
 *==================================================================================================================*/
;(function($) {
	'use strict';
	
    $.fn.bPopup = function(options, callback) {
        
    	if ($.isFunction(options)) {
            callback 		= options;
            options 		= null;
        }

		// OPTIONS
        var o 				= $.extend({}, $.fn.bPopup.defaults, options);
		
		// HIDE SCROLLBAR?  
        if (!o.scrollBar)
            $('html').css('overflow', 'hidden');
        
		// VARIABLES	
        var $popup 			= this
          , d 				= $(document)
          , w 				= window
		  , $w				= $(w)
          , wH				= windowHeight()
		  , wW				= windowWidth()
          , prefix			= '__b-popup'
		  , isIOS6X			= (/OS 6(_\d)+/i).test(navigator.userAgent) // Used for a temporary fix for ios6 timer bug when using zoom/scroll 
          , buffer			= 200
		  , popups			= 0
          , id
          , inside
          , fixedVPos
          , fixedHPos
          , fixedPosStyle
		  , vPos
          , hPos
		  , height
		  , width
		  , debounce
		  , autoCloseTO
		;

		////////////////////////////////////////////////////////////////////////////////////////////////////////////
        // PUBLIC FUNCTION - call it: $(element).bPopup().close();
		////////////////////////////////////////////////////////////////////////////////////////////////////////////
        $popup.close = function() {
            close();
        };
		
        $popup.reposition = function(animateSpeed) {
            reposition(animateSpeed);
        };

        return $popup.each(function() {
            if ($(this).data('bPopup')) return; //POPUP already exists?
            init();
        });

        ////////////////////////////////////////////////////////////////////////////////////////////////////////////
        // HELPER FUNCTIONS - PRIVATE
        ////////////////////////////////////////////////////////////////////////////////////////////////////////////
        function init() {
            triggerCall(o.onOpen);
			popups = ($w.data('bPopup') || 0) + 1, 
			id = prefix + popups + '__',
			fixedVPos = o.position[1] !== 'auto', 
			fixedHPos = o.position[0] !== 'auto', 
			fixedPosStyle = o.positionStyle === 'fixed', 
			height = $popup.outerHeight(true), 
			width = $popup.outerWidth(true);
			
            o.loadUrl ? createContent() : open();
        };
		
		function createContent() {
            o.contentContainer = $(o.contentContainer || $popup);
            switch (o.content) {
                case ('iframe'):
					var iframe = $('<iframe class="b-iframe" ' + o.iframeAttr +'></iframe>');
					iframe.appendTo(o.contentContainer);
					height = $popup.outerHeight(true);
					width = $popup.outerWidth(true);
					open();
					iframe.attr('src', o.loadUrl); // setting iframe src after open due IE9 bug
					triggerCall(o.loadCallback);
                    break;
				case ('image'):
					open();
					$('<img />')
						.load(function() {
						    triggerCall(o.loadCallback);
							recenter($(this));
					    }).attr('src', o.loadUrl).hide().appendTo(o.contentContainer);
					break;
                default:
					open();
					$('<div class="b-ajax-wrapper"></div>')
                    	.load(o.loadUrl, o.loadData, function(response, status, xhr) {
						    triggerCall(o.loadCallback, status);
							recenter($(this));
						}).hide().appendTo(o.contentContainer);
                    break;
            }
        };

		function open(){
			// MODAL OVERLAY
            if (o.modal) {
                $('<div class="b-modal '+id+'"></div>')
                .css({backgroundColor: o.modalColor, position: 'fixed', top: 0, right:0, bottom:0, left: 0, opacity: 0, zIndex: o.zIndex + popups})
                .appendTo(o.appendTo)
                .fadeTo(o.speed, o.opacity);
            }
			
			// POPUP
			calcPosition();
			
			var _xPosVal = o.transition == 'slideIn' || o.transition == 'slideBack' ? (o.transition == 'slideBack' ? d.scrollLeft() + wW : (hPos + width) *-1) : getLeftPos(!(!o.follow[0] && fixedHPos || fixedPosStyle));
			var _yPosVal = o.transition == 'slideDown' || o.transition == 'slideUp' ? (o.transition == 'slideUp' ? d.scrollTop() + wH : vPos + height * -1) : getTopPos(!(!o.follow[1] && fixedVPos || fixedPosStyle));
			
			$popup
				.data('bPopup', o).data('id',id)
				.css({ 
						'position': o.positionStyle || 'absolute',
						'z-index': o.zIndex + popups + 1 
				});
			
			if ( o.position[0] == -1 ) {
				$popup.css({ 
						'right': _xPosVal,
				});
			} else {
				$popup.css({ 
						'left': _xPosVal,
				});				
			}
			if ( o.position[1] == -1 ) {
				$popup.css({ 
						'bottom': _yPosVal,
				});
			} else {
				$popup.css({ 
						'top': _yPosVal,
				});				
			}
			$popup.each(function() {
            		if(o.appending) {
                		$(this).appendTo(o.appendTo);
            		}
        		});
			
			
            // $popup
				// .data('bPopup', o).data('id',id)
				// .css({ 
						// 'left': _xPosVal,
						// 'position': o.positionStyle || 'absolute',
						// 'top': _yPosVal,
						// 'z-index': o.zIndex + popups + 1 
				// }).each(function() {
            		// if(o.appending) {
                		// $(this).appendTo(o.appendTo);
            		// }
        		// });
				
				
			doTransition(true);	
		};
		
        function close() {
            if (o.modal) {
                $('.b-modal.'+$popup.data('id'))
	                .fadeTo(o.speed, 0, function() {
	                    $(this).remove();
	                });
            }
			// Clean up
			unbindEvents();	
			clearTimeout(autoCloseTO);
			// Close trasition
            doTransition();
            
			return false; // Prevent default
        };
		
		function reposition(animateSpeed){
            wH = windowHeight();
  		    wW = windowWidth();
			inside = insideWindow();
           	if(inside.x || inside.y){
				clearTimeout(debounce);
				debounce = setTimeout(function(){
					calcPosition();
					animateSpeed = animateSpeed || o.followSpeed;
					var css = {};
					if(inside.x) {
						if ( o.position[0] == -1 ) {
							css.right = o.follow[0] ? getLeftPos(true) : 'auto';
						} else {
							css.left = o.follow[0] ? getLeftPos(true) : 'auto';				
						}						
					}
					if(inside.y) {
						if ( o.position[1] == -1 ) {
							css.bottom = o.follow[1] ? getTopPos(true) : 'auto';
						} else {
							css.top = o.follow[1] ? getTopPos(true) : 'auto';			
						}						
					}
					$popup
                       	.dequeue()
                       	.each(function() {
                           	if(fixedPosStyle) {
								if ( o.position[0] == -1 ) {
									$(this).css({'right': hPos});
								} else {
									$(this).css({'left': hPos});				
								}
								if ( o.position[1] == -1 ) {
									$(this).css({'bottom': vPos});
								} else {
									$(this).css({'top': vPos});				
								}								
                           	}
                           	else {
                               	$(this).animate(css, animateSpeed, o.followEasing);
                           	}
                       	});
				}, 50);					
           	}
		};
		
		//Eksperimental
		function recenter(content){
			var _width = content.width(), _height = content.height(), css = {};
			o.contentContainer.css({height:_height,width:_width});

			if (_height >= $popup.height()){
				css.height = $popup.height();
			}
			if(_width >= $popup.width()){
				css.width = $popup.width();
			}
			height = $popup.outerHeight(true)
			, width = $popup.outerWidth(true);
				
			calcPosition();
			o.contentContainer.css({height:'auto',width:'auto'});		
			
			css.left = getLeftPos(!(!o.follow[0] && fixedHPos || fixedPosStyle)),
			css.top = getTopPos(!(!o.follow[1] && fixedVPos || fixedPosStyle));

			$popup
				.animate(
					css
					, 250
					, function() { 
						content.show();
						inside = insideWindow();
					}
				);
		};
		
        function bindEvents() {
            $w.data('bPopup', popups);
			$popup.delegate('.bClose, .' + o.closeClass, 'click.'+id, close); // legacy, still supporting the close class bClose
            
            if (o.modalClose) {
                $('.b-modal.'+id).css('cursor', 'pointer').bind('click', close);
            }
			
			// Temporary disabling scroll/resize events on devices with IOS6+
			// due to a bug where events are dropped after pinch to zoom
            if (!isIOS6X && (o.follow[0] || o.follow[1])) {
               $w.bind('scroll.'+id, function() {
                	if(inside.x || inside.y){
						var css = {};
						if(inside.x) {
							if ( o.position[0] == -1 ) {
								css.right = o.follow[0] ? getLeftPos(!fixedPosStyle) : 'auto';
							} else {
								css.left = o.follow[0] ? getLeftPos(!fixedPosStyle) : 'auto';
							}
						}
						if(inside.y) {
							if ( o.position[1] == -1 ) {
								css.bottom = o.follow[1] ? getTopPos(!fixedPosStyle) : 'auto';
							} else {
								css.top = o.follow[1] ? getTopPos(!fixedPosStyle) : 'auto';
							}							
						}
                    	$popup
                        	.dequeue()
                            .animate(css, o.followSpeed, o.followEasing);
					 }  
            	}).bind('resize.'+id, function() {
		            reposition();
                });
            }
            if (o.escClose) {
                d.bind('keydown.'+id, function(e) {
                    if (e.which == 27) {  //escape
                        close();
                    }
                });
            }
        };
		
        function unbindEvents() {
            if (!o.scrollBar) {
                $('html').css('overflow', 'auto');
            }
            $('.b-modal.'+id).unbind('click');
            d.unbind('keydown.'+id);
            $w.unbind('.'+id).data('bPopup', ($w.data('bPopup')-1 > 0) ? $w.data('bPopup')-1 : null);
            $popup.undelegate('.bClose, .' + o.closeClass, 'click.'+id, close).data('bPopup', null);
        };
		
		function doTransition(open) {
			switch (open ? o.transition : o.transitionClose || o.transition) {
			   case "slideIn":
					if ( o.position[0] == -1 ) {
						animate({
							right: open ? getLeftPos(!(!o.follow[0] && fixedHPos || fixedPosStyle)) : d.scrollLeft() - (width || $popup.outerWidth(true)) - buffer
						});
					} else {
						animate({
							left: open ? getLeftPos(!(!o.follow[0] && fixedHPos || fixedPosStyle)) : d.scrollLeft() - (width || $popup.outerWidth(true)) - buffer
						});
					}	   
			      	break;
			   case "slideBack":
					if ( o.position[0] == -1 ) {
						animate({
							right: open ? getLeftPos(!(!o.follow[0] && fixedHPos || fixedPosStyle)) : d.scrollLeft() + wW + buffer
						});
					} else {
						animate({
							left: open ? getLeftPos(!(!o.follow[0] && fixedHPos || fixedPosStyle)) : d.scrollLeft() + wW + buffer
						});
					}	   			   
			      	break;
			   case "slideDown":
					if ( o.position[1] == -1 ) {
						animate({
							bottom: open ? getTopPos(!(!o.follow[1] && fixedVPos || fixedPosStyle)) : d.scrollTop() - (height || $popup.outerHeight(true)) - buffer							
						});
					} else {
						animate({
							top: open ? getTopPos(!(!o.follow[1] && fixedVPos || fixedPosStyle)) : d.scrollTop() - (height || $popup.outerHeight(true)) - buffer
						});
					}
			      	break;
		   		case "slideUp":
					if ( o.position[1] == -1 ) {
						animate({
							bottom: open ? getTopPos(!(!o.follow[1] && fixedVPos || fixedPosStyle)) : d.scrollTop() + wH + buffer
						});
					} else {
						animate({
							top: open ? getTopPos(!(!o.follow[1] && fixedVPos || fixedPosStyle)) : d.scrollTop() + wH + buffer
						});
					}					
		      	  	break;
			   default:
			   	  	//Hardtyping 1 and 0 to ensure opacity 1 and not 0.9999998
				  	$popup.stop().fadeTo(o.speed, open ? 1 : 0, function(){onCompleteCallback(open);});
			}
			
			function animate(css){
			  	$popup
					.css({display: 'block',opacity: 1})
					.animate(css, o.speed, o.easing, function(){ onCompleteCallback(open); });
			};
		};
		
		
		function onCompleteCallback(open){
			if(open){
				bindEvents();
	            triggerCall(callback);
				if(o.autoClose){
					autoCloseTO = setTimeout(close, o.autoClose);
				}
			} else {
				$popup.hide();
				triggerCall(o.onClose);
				if (o.loadUrl) {
                    o.contentContainer.empty();
					$popup.css({height: 'auto', width: 'auto'});
                }		
			}
		};
		
		function getLeftPos(includeScroll){
			return includeScroll ? hPos + d.scrollLeft() : hPos;
		};
		
		function getTopPos(includeScroll){
			return includeScroll ? vPos + d.scrollTop() : vPos;
		};
		
		function triggerCall(func, arg) {
			$.isFunction(func) && func.call($popup, arg);
		};
		
       	function calcPosition(){
			vPos 		= fixedVPos ? o.position[1] : Math.max(0, ((wH- $popup.outerHeight(true)) / 2) - o.amsl)
			, hPos 		= fixedHPos ? o.position[0] : (wW - $popup.outerWidth(true)) / 2
			, inside 	= insideWindow();
		};
		
        function insideWindow(){
            return {  
				x: wW > $popup.outerWidth(true),
				y: wH > $popup.outerHeight(true)	
			}
        };
		
		function windowHeight(){
			return $w.height();
		};
		
		function windowWidth(){
			return $w.width();
		};
    };

	////////////////////////////////////////////////////////////////////////////////////////////////////////////
	// DEFAULT VALUES
	////////////////////////////////////////////////////////////////////////////////////////////////////////////
    $.fn.bPopup.defaults = {
          amsl: 			50
        , appending: 		true
        , appendTo: 		'body'
		, autoClose:		false
        , closeClass: 		'b-close'
        , content: 			'ajax' // ajax, iframe or image
        , contentContainer: false
		, easing: 			'swing'
        , escClose: 		true
        , follow: 			[true, true] // x, y
		, followEasing: 	'swing'
        , followSpeed: 		500
		, iframeAttr: 		'scrolling="no" frameborder="0"'
		, loadCallback: 	false
		, loadData: 		false
        , loadUrl: 			false
        , modal: 			true
        , modalClose: 		true
        , modalColor: 		'#000'
        , onClose: 			false
        , onOpen: 			false
        , opacity: 			0.7
        , position: 		['auto', 'auto'] // x, y,
        , positionStyle: 	'absolute'// absolute or fixed
        , scrollBar: 		true
		, speed: 			250 // open & close speed
		, transition:		'fadeIn' //transitions: fadeIn, slideDown, slideIn, slideBack
		, transitionClose:	false
        , zIndex: 			9997 // popup gets z-index 9999, modal overlay 9998
    };
})(jQuery);

/*================================================================================
 * Config
================================================================================*/
function LoadESPopup(key) 
{
	// Settings
	var popupSettings = ips.getSetting('popup_' + key);
	
	// Setup Responsive Layout
	$(window).on('resize', function() {
		if ( $(window).width() >= popupSettings['width'] ) {
			$('#espopup_' + key).css({
				"width": popupSettings['width'] + "px", 
				"height": popupSettings['height'] + "px"
			});
			$('#popupcontent_' + key).css({
				"height": popupSettings['height'] + "px"
			});					
		}
		else {
			$('#espopup_' + key + "," + "#popupcontent_" + key).css({
				"width": "100%", 
				"height": ( $(window).width() * popupSettings['height'] ) / popupSettings['width'] + "px"
			});	
		}
	}).resize();
	
	// Open Popup
	if (popupSettings['notshowafterclose'] == 1) {
		if ( ips.utils.cookie.get( 'espopup_notshow_' + key ) ) {
			return;
		}
	}
	
	if (popupSettings['showtime'] > 0) {
		if ( ips.utils.cookie.get( 'espopup_' + key ) ) { 
			return; 
		}
	}
	
	if (popupSettings['showpagecount'] != 0) {
		if ( !ips.utils.cookie.get( 'espp_pageview_' + key ) ) {
			if (popupSettings['pageview'] != 1) {
				ips.utils.cookie.set( 'espp_pageview_' + key, 1 ); 
				return;
			}
			else {
				ips.utils.cookie.set( 'espp_pageview_' + key, 2 );
			}
		}
		else {
			
			if ( popupSettings['showpagecount'].indexOf("+") < 0 ) {
				if ( parseInt( ips.utils.cookie.get( 'espp_pageview_' + key ) ) > popupSettings['pageview'] ) { 
					return; 
				}
			}
			if ( parseInt( ips.utils.cookie.get( 'espp_pageview_' + key ) ) < popupSettings['pageview'] )
			{
				ips.utils.cookie.set( 'espp_pageview_' + key, parseInt( ips.utils.cookie.get( 'espp_pageview_' + key ) ) + 1 );
				return;
			}
			else
			{
				if ( popupSettings['showpagecount'].indexOf("+") < 0 ) {
					ips.utils.cookie.set( 'espp_pageview_' + key, parseInt( ips.utils.cookie.get( 'espp_pageview_' + key ) ) + 1 );
				}
				else {
					ips.utils.cookie.set( 'espp_pageview_' + key, 0 );
				}
			}
		}
	}
	
	if ( $('.b-HardClose').length > 0 ) {
		$('.b-HardClose').click(function() {
			var key = $(this).attr("id");
			ips.utils.cookie.set( 'espopup_notshow_' + key, 1 );
		});
	}
	
	$('#espopup_' + key).bPopup({
		appendTo: 'body', 
		zIndex: 10000, 
		position: popupSettings['position'] ? popupSettings['position'] : ['auto','auto'],
		modalColor: popupSettings['modalcolor'] ? popupSettings['modalcolor'] : '#000',			
		speed: popupSettings['speed'],
		positionStyle: 'fixed',
		transition: popupSettings['transitionOpen'],
		transitionClose: popupSettings['transitionClose'],	
		scrollBar: popupSettings['scrollBar'],				
		modalClose: popupSettings['modalClose'],
		opacity: popupSettings['opacity'],
		autoClose: popupSettings['autoclose'],
		onOpen: function() {
			if (popupSettings['autoclose'] > 0) {
				countDownTimer( key, popupSettings['autoclose'] );	
			}
		}, 
		onClose: function() {
			$('#espopup_' + key).html(''); 
			if (popupSettings['showtime'] > 0) {
				ppSetCookie( key, popupSettings['showtime'] );
			}
			if (popupSettings['redirect'] && popupSettings['redirect'] != popupSettings['url'] ) {
				window.location.href = popupSettings['redirect'];
			}
		}			
	});
	
	$('#popupcontent_' + key).html( ppDecodeEntities( popupSettings['content'] ) );
}


function countDownTimer(id, minutes) {
	var seconds = minutes/1000;
	var counter = seconds;
	setInterval(function() {
		counter--;
		if ( counter  >= 0) { $("#esppclose_" + id).html( counter  ); }
		if ( counter  === 0) { clearInterval( counter ); }
	}, 1000);	
}

function ppSetCookie(id, showtime) {
	var date = new Date();
	date.setTime( date.getTime() + ( showtime * 60 * 1000 ) );	// minute			
	ips.utils.cookie.set( 'espopup_' + id, true, date.toUTCString() );
}

function ppDecodeEntities(str) {
    return jQuery('<div></div>').html(str).text();
}
