<?php

namespace IPS\easypopup;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * Featured Content Node
 */
class _Popup extends \IPS\Node\Model
{
	/**
	 * @brief	[ActiveRecord] Multiton Store
	 */
	protected static $multitons;
		
	/**
	 * @brief	[ActiveRecord] Default Values
	 */
	protected static $defaultValues = NULL;
	
	/**
	 * @brief	[ActiveRecord] Database Table
	 */
	public static $databaseTable = 'easypopup_popups';
	
	/**
	 * @brief	[ActiveRecord] Database Prefix
	 */
	public static $databasePrefix = 'espp_';
		
	/**
	 * @brief	[Node] Order Database Column
	 */
	public static $databaseColumnOrder = 'order';
	
	/**
	 * @brief	[Node] Node Title
	 */
	public static $nodeTitle = 'es_popup';
			
	/**
	 * @brief	[Node] Show forms modally?
	 */
	public static $modalForms = FALSE;
	
	/**
	 * @brief	[Node] ACP Restrictions
	 * @code
	 	array(
	 		'app'		=> 'core',				// The application key which holds the restrictrions
	 		'module'	=> 'foo',				// The module key which holds the restrictions
	 		'map'		=> array(				// [Optional] The key for each restriction - can alternatively use "prefix"
	 			'add'			=> 'foo_add',
	 			'edit'			=> 'foo_edit',
	 			'permissions'	=> 'foo_perms',
	 			'delete'		=> 'foo_delete'
	 		),
	 		'all'		=> 'foo_manage',		// [Optional] The key to use for any restriction not provided in the map (only needed if not providing all 4)
	 		'prefix'	=> 'foo_',				// [Optional] Rather than specifying each  key in the map, you can specify a prefix, and it will automatically look for restrictions with the key "[prefix]_add/edit/permissions/delete"
	 * @encode
	 */
	protected static $restrictions = array(
		'app'		=> 'easypopup',
		'module'	=> 'easypopup',
		'prefix' 	=> 'easypopup_',
		'all'		=> 'easypopup_manage',
	);
	

	/**
	 * @brief	[Node] Prefix string that is automatically prepended to permission matrix language strings
	 */
	public static $permissionLangPrefix = 'esppPerm_';
	
	/**
	 * [Node] Get whether or not this node is enabled
	 *
	 * @note	Return value NULL indicates the node cannot be enabled/disabled
	 * @return	bool|null
	 */
	protected function get__enabled()
	{
		return $this->open;
	}

	/**
	 * [Node] Set whether or not this node is enabled
	 *
	 * @param	bool|int	$enabled	Whether to set it enabled or disabled
	 * @return	void
	 */
	protected function set__enabled( $enabled )
	{
		$this->open	= $enabled;
	}	
	
	/**
	 * [Node] Get Node Title
	 *
	 * @return	string
	 */
	protected function get__title()
	{
		return $this->title;
	}
	
	protected static $bypassFilters = array(
		'core_Group',
		'bouncer_Disabled',
		'core_Name'
	);
	
	/**
	 * [Node] Add/Edit Form
	 *
	 * @param	\IPS\Helpers\Form	$form	The form
	 * @return	void
	 */
	public function form( &$form )
	{
		# Basic
		$form->addTab( 'espp_tab_basic' );		
		$form->add( new \IPS\Helpers\Form\Text( 'espp_title', $this->id ? $this->title : "", TRUE, array( 'maxLength' => 255 ) ) );
		$form->add( new \IPS\Helpers\Form\Number( 'espp_w', $this->id ? $this->w : 500, TRUE ) );
		$form->add( new \IPS\Helpers\Form\Number( 'espp_h', $this->id ? $this->h : 350, TRUE ) );
		
		$posOptions = array( 
			"0,0" => "espp_pos_top_left",
			"'auto', 0" => "espp_pos_top_center",
			"-1, 0" => "espp_pos_top_right",
			"0, 'auto'" => "espp_pos_center_left",
			"'auto','auto'" => "espp_pos_center",
			"-1,'auto'" => "espp_pos_center_right",
			"0,-1" => "espp_pos_bottom_left",
			"'auto',-1" => "espp_pos_bottom_center",
			"-1,-1" => "espp_pos_bottom_right",
		);
		
		$form->add( new \IPS\Helpers\Form\Select( 'espp_position', $this->position ? $this->position : "", TRUE, array('options' => $posOptions) ) );		
			
		$form->addTab( 'espp_tab_content' );
		$options = array( 'html' => 'espp_type_html', 'image' => 'espp_type_image' );
		$toggles = array( 'html' => array( 'espp_content1' ), 'image' => array( 'espp_img_linkto', 'espp_image', 'espp_link_newtab' ) );
		$form->add( new \IPS\Helpers\Form\Select( 'espp_content_type', $this->id ? $this->content_type : 'html', TRUE, array( 'options' => $options, 'toggles' => $toggles ) ) );
		
		if ( \IPS\Settings::i()->espp_useEditor != 1 )
		{
			$form->add( new \IPS\Helpers\Form\Codemirror( 'espp_content', $this->content ? $this->content : "", FALSE, array(), NULL, NULL, NULL, 'espp_content1' ) );
		}
		else
		{
			$autoSaveKey = $this->id ? "EasyPopup_editor_{$this->id}" : "EasyPopup_editor_new";
			$form->add( new \IPS\Helpers\Form\Editor( 'espp_content', $this->id ? $this->content : NULL, FALSE, array( 'app' => 'easypopup', 'key' => 'EasyPopup', 'autoSaveKey' => $autoSaveKey, 'attachIds' => ( $this === NULL ? NULL : array( $this->id ) ) ), NULL, NULL, NULL, 'espp_content1' ) );		
		}
		
		$form->add( new \IPS\Helpers\Form\Url( 'espp_img_linkto', $this->id ? $this->img_linkto : NULL, FALSE, array(), NULL, NULL, NULL, 'espp_img_linkto' ) );
		$form->add( new \IPS\Helpers\Form\Upload( 'espp_image', $this->id ? \IPS\File::get( 'easypopup_Image', $this->image ) : NULL, FALSE, array( 'image' => TRUE, 'storageExtension' => 'easypopup_Image' ), NULL, NULL, NULL, 'espp_image' ) );
		$form->add( new \IPS\Helpers\Form\YesNo( 'espp_link_newtab', $this->id ? $this->link_newtab : 1, FALSE, array(), NULL, NULL, NULL, 'espp_link_newtab' ) );
		
		# Popup
		$form->addTab( 'espp_tab_popup' );		
		$form->addHeader( 'espp_tab_popup' );			
		$form->add( new \IPS\Helpers\Form\Select( 'espp_transitionOpen', $this->id ? $this->transitionOpen : "fadeIn", FALSE, array(
			'options' => array(
				'fadeIn' 		=> 'fadeIn',
				'slideDown' 	=> 'slideDown',
				'slideUp' 		=> 'slideUp',
				'slideIn' 		=> 'slideIn',
				'slideBack' 	=> 'slideBack',			
			) ) ) );			
		$form->add( new \IPS\Helpers\Form\Select( 'espp_transitionClose', $this->id ? $this->transitionClose : "fadeIn", FALSE, array(
			'options' => array(
				'fadeIn' 		=> 'fadeOut',
				'slideDown' 	=> 'slideDown',
				'slideUp' 		=> 'slideUp',
				'slideIn' 		=> 'slideIn',
				'slideBack' 	=> 'slideBack',			
			) ) ) );		

		$form->add( new \IPS\Helpers\Form\Number( 'espp_speed', ($this->id and $this->speed > 0) ? $this->speed : 250, FALSE ) );
		$form->add( new \IPS\Helpers\Form\YesNo( 'espp_scrollBar', $this->id ? $this->scrollBar : 0, FALSE ) );		
		$form->add( new \IPS\Helpers\Form\Color( 'espp_modalcolor', $this->id ? $this->modalcolor : '#000', FALSE ) );	
		$form->add( new \IPS\Helpers\Form\Number( 'espp_modalopacity', $this->id && $this->modalopacity ? $this->modalopacity : 0.7, FALSE, array( 'decimals' => 1 ) ) );	
		$form->add( new \IPS\Helpers\Form\Text( 'espp_bg', $this->id ? $this->bg : '#fff' ) );	
		$form->add( new \IPS\Helpers\Form\Number( 'espp_autoclose', ($this->id and $this->autoclose > 0) ? $this->autoclose : 0, FALSE ) );	
		$form->add( new \IPS\Helpers\Form\YesNo( 'espp_modalClose', $this->id ? $this->modalClose : 1, FALSE ) );				
		$form->add( new \IPS\Helpers\Form\YesNo( 'espp_closebutton', $this->id ? $this->closebutton : 1, FALSE, array( 'togglesOn' => array('espp_notshowafterclose') ) ) );		
		$form->add( new \IPS\Helpers\Form\YesNo( 'espp_notshowafterclose', $this->id ? $this->notshowafterclose : 0, FALSE, array(), NULL, NULL, NULL, 'espp_notshowafterclose' ) );		
		$form->addHeader( 'espp_tab_redirect' );		
		$form->add( new \IPS\Helpers\Form\Url( 'espp_redirect', $this->redirect ? $this->redirect : null, FALSE, array() ) );
		
		# Timer
		$form->addTab( 'espp_tab_time' );				
		$form->add( new \IPS\Helpers\Form\Number( 'espp_wait', $this->id ? $this->wait : 0, FALSE ) );			
		$form->add( new \IPS\Helpers\Form\Number( 'espp_showtime', $this->id ? $this->showtime : 0, FALSE ) );		
		$form->add( new \IPS\Helpers\Form\Text( 'espp_showpagecount', $this->id ? $this->showpagecount : 0, FALSE ) );	
		
		# Page Filter
		$form->addTab( 'espp_tab_page_flt' );
		$form->add( new \IPS\Helpers\Form\Select( 'espp_pagetype', $this->id ? $this->pagetype : "include", FALSE, array(
			'options' => array(
				'include' 	=> 'espp_include',
				'exclude' 	=> 'espp_exclude',
			) ) ) );		
		$form->add( new \IPS\Helpers\Form\TextArea( 'espp_pagelist', $this->pagelist ? $this->pagelist : "*", FALSE, array( 'rows' => 5 ) ) );	
		$form->add( new \IPS\Helpers\Form\TextArea( 'espp_alwayexclude', $this->alwayexclude ? $this->alwayexclude : null, FALSE, array( 'rows' => 5 ) ) );	
	
	
		# Member filter
		$form->addTab( 'espp_tab_mem_flt' );	
		$form->add( new \IPS\Helpers\Form\Select( 'espp_perm_groups', $this->id ? explode( ',', $this->perm_groups ) : null, FALSE, array( 'options' => \IPS\Member\Group::groups(), 'multiple' => TRUE, 'parse' => 'normal' ) ) );			
		$form->add( new \IPS\Helpers\Form\Select( 'espp_perm_exgroups', $this->id ? explode( ',', $this->perm_exgroups ) : null, FALSE, array( 'options' => \IPS\Member\Group::groups(), 'multiple' => TRUE, 'parse' => 'normal' ) ) );			
		
		$lastApp	= 'core';

		foreach ( \IPS\Application::allExtensions( 'core', 'MemberFilter', TRUE, 'core' ) as $key => $extension )
		{
			if( method_exists( $extension, 'getSettingField' ) AND !in_array( $key, self::$bypassFilters ) )
			{
				/* See if we need a new form header - one per app */
				$_key		= explode( '_', $key );
				
				if( $_key[0] != $lastApp )
				{
					$lastApp	= $_key[0];
					$form->addHeader( $lastApp . '_bm_filters' );
				}

				/* Grab our fields and add to the form */
				$fields		= $extension->getSettingField( !empty( $this->_options[ $key ] ) ? $this->_options[ $key ] : array() );

				foreach( $fields as $field )
				{
					$form->add( $field );
				}
			}
		}
	}
	
	/**
	 * [Node] Save Add/Edit Form
	 *
	 * @param	array	$values	Values from the form
	 * @return	void
	 */
	public function formatFormValues( $values )
	{
		if ( !$this->id )
		{
			$this->save();
		}	
		
		$_options	= array();

		foreach ( \IPS\Application::allExtensions( 'core', 'MemberFilter', TRUE, 'core' ) as $key => $extension )
		{
			if( method_exists( $extension, 'save' ) AND !in_array( $key, self::$bypassFilters ) )
			{
				$_value		= $extension->save( $values );

				if( $_value )
				{
					$_options[ $key ]	= $_value;
				}
			}
		}
		
		$values['espp_opts'] = json_encode( $_options );
		
		foreach( $values as $k => $v )
		{
			if( mb_stripos($k, 'espp_') === false )
			{
				unset( $values[ $k ] );
			}
		}

		return parent::formatFormValues( $values );
	}

	/**
	 * [ActiveRecord] Delete Record
	 *
	 * @return	void
	 */
	public function delete()
	{
		parent::delete();
		if ( $this->image )
		{
			try
			{			
				\IPS\File::get( 'easypopup_Image', $this->image )->delete();
			}
			catch( \Exception $e ){}
		}
	}
	
	/**
	 * Search
	 *
	 * @param	string		$column	Column to search
	 * @param	string		$query	Search query
	 * @param	string|null	$order	Column to order by
	 * @param	mixed		$where	Where clause
	 * @return	array
	 */
	public static function search( $column, $query, $order=NULL, $where=array() )
	{
		if ( $column === '_title' )
		{
			$column	= 'espp_title';
		}

		if( $order == '_title' )
		{
			$order	= 'espp_title';
		}

		return parent::search( $column, $query, $order, $where );
	}
	

	/**
	 * [Node] Get buttons to display in tree
	 * Example code explains return value
	 * @endcode
	 * @param	string	$url		Base URL
	 * @param	bool	$subnode	Is this a subnode?
	 * @return	array
	 */
	public function getButtons( $url, $subnode=FALSE )
	{
		$buttons = parent::getButtons( $url, $subnode );

		$buttons['reset'] = array(
				'icon'	   => 'refresh',
				'title'    => 'espp_reset',
				'link'	   => $url->setQueryString( array( 'do' => 'resetkey', 'id' => $this->id ) ),
		);

		return $buttons;
	}

	/**
	 * Get the associated club
	 *
	 * @return	\IPS\Member\Club|NULL
	 */
	public function club()
	{
		return NULL;
	}	
	
	/**
	 * Get the options
	 *
	 * @return	array
	 */
	public function get__options()
	{
		if ( !isset( $this->_data['opts'] ) )
		{
			return array();
		}
		
		return json_decode( $this->_data['opts'], TRUE );
	}

	/**
	 * Set the options
	 *
	 * @param	array	$value options
	 * @return	vpid
	 */
	public function set__options( $value )
	{
		$this->opts	= json_encode( $value );
	}
	
	/**
	* WHO CAN SEE
	*/	
	public function canSeePopup()
	{
		if ( !$this->checkPageToDisplay() )
		{
			return FALSE;
		}

		if ( $this->perm_exgroups && \IPS\Member::loggedIn()->inGroup( explode( ",", $this->perm_exgroups ) ) )
		{
			return FALSE;
		}

		if ( $this->perm_groups &&  ! \IPS\Member::loggedIn()->inGroup( explode( ",", $this->perm_groups ) ) )
		{
			return FALSE;
		}
		
		if ( $this->getQuery() == 0 )
		{
			return FALSE;
		}
		
		return TRUE;
	}
	 
	protected function getQuery()
	{
		/* Compile where */
		$where = array();
	
		foreach ( \IPS\Application::allExtensions( 'core', 'MemberFilter', FALSE, 'core' ) as $key => $extension )
		{
			if( method_exists( $extension, 'getQueryWhereClause' ) AND !in_array( $key, self::$bypassFilters ) )
			{
				/* Grab our fields and add to the form */
				if( !empty( $this->_options[ $key ] ) )
				{
					if( $_where = $extension->getQueryWhereClause( $this->_options[ $key ] ) )
					{
						if ( is_string( $_where ) )
						{
							$_where = array( $_where );
						}
							
						$where	= array_merge( $where, $_where );
					}
				}
			}
		}
		
		if ( count( $where ) > 0 )
		{
			if ( \IPS\Member::loggedIn()->member_id )
			{
				$where[] = array( "core_members.member_id=" . \IPS\Member::loggedIn()->member_id );
			
				/* Compile query */
				$query = \IPS\Db::i()->select( 'COUNT(*)', 'core_members', $where, NULL, 1 );
				
				/* Run callbacks */
				foreach ( \IPS\Application::allExtensions( 'core', 'MemberFilter', FALSE, 'core' ) as $key => $extension )
				{
					if( method_exists( $extension, 'queryCallback' ) AND !in_array( $key, self::$bypassFilters ))
					{
						/* Grab our fields and add to the form */
						if( !empty( $this->_options[ $key ] ) )
						{
							$data = $this->_options[ $key ];
							$extension->queryCallback( $data, $query );
						}
					}
				}

				return $query->count();
			}
			else
			{
				return 0;
			}
		}
		
		return -1;
	}
	
	protected function checkPageToDisplay()
	{
		$current_page = (string) \IPS\Request::i()->url();
		
		if ( $this->alwayexclude )
		{
			$expage = explode( "\n", str_replace( "\r", "", $this->alwayexclude ) );
			foreach ( $expage as $exurl )
			{
				if ( preg_match( "~^" . str_replace( '\*', '.*?', preg_quote( $exurl ) ) . "$~i", htmlspecialchars_decode( $current_page ) ) )
				{
					return false;
				}				
				if ( $exurl == $current_page )
				{
					return false;
				}
			}			
		}
		
		if ( $this->pagelist == '*' )
		{
			return $this->pagetype == 'include' ? true : false;
		}	
		
		$page_list = explode( "\n", str_replace( "\r", "", $this->pagelist ) );
		
		if ( !count( $page_list ) )
		{
			return false;
		}
		if ( $this->pagetype == 'include' )
		{
			foreach ( $page_list as $url )
			{
				if ( preg_match( "~^" . str_replace( '\*', '.*?', preg_quote( $url ) ) . "$~i", htmlspecialchars_decode( $current_page ) ) )
				{
					return true;
				}				
				if ( $url == $current_page )
				{
					return true;
				}
			}
			return false;
		}
		else
		{
			foreach ( $page_list as $url )
			{
				if ( preg_match( "~^" . str_replace( '\*', '.*?', preg_quote( $url ) ) . "$~i", htmlspecialchars_decode( $current_page ) ) )
				{
					return false;
				}				
				if ( $url == $current_page )
				{
					return false;
				}
			}
			return true;
		}
		
		return false;
	}		 
}